<?php

require(INC_DIR .'classes/seoUtf8.php');

class Link
{
	protected static $title = null;
	protected static $id = null;
	protected static $index = 'index.php';
	protected static $_request;
	protected static $_response;

	private static function replaceUrl($url, $param) 
	{
		return strtr($url, $param);
	}
	
	private static function getTitleForUrl()
	{
		global $bb_cfg;
		
		$title = strval(preg_replace('/&(.+?);/', '', self::$title));
		
		if ($bb_cfg['romanize'])
		{
			$title = utf8_romanize(utf8_deaccent($title));
		}

		$title = strtr(
			$title,
			'`!"$%^&*()-+={}[]<>;:@#~,./?|' . "\r\n\t\\",
			'                             ' . '    '
		);
		$title = self::replaceUrl($title, array('"' => '', "'" => ''));
		
		if ($bb_cfg['romanize'])
		{
			$title = preg_replace('/[^a-zA-Z0-9_ -]/', '', $title);
		}

		$title = preg_replace('/[ ]+/', '-', trim($title));

		return mb_strtolower($title);
	}
	
	public static function BuildUrl($mode = false, $id = false, $title = false)
	{
		global $bb_cfg;
		
		self::$title = $title;
		self::$id = $id;
		$url = ($mode) ? self::url($mode) : make_url('');
		
		return $url;
	}
	
	private static function url($mode)
	{
		global $bb_cfg;
		
		if(!self::$id) {	
			$ret = $mode;		
		} elseif(!self::$title) {
			$ret = $mode.'/'.intval(self::$id);
		} else {	
			$ret = $mode.'/'.self::getTitleForUrl().'.'.intval(self::$id);
		}
		
		$prefix = ($bb_cfg['no_rewite_url']) ? '' : self::$index.'?';
		
		return $prefix.$ret.'/';
	}
	
	public static function requestUrl()
	{
		return ($_SERVER['REQUEST_URI']) ? htmlspecialchars(stripslashes(trim($_SERVER['REQUEST_URI']))) : '';
	}

	public static function detectedUrl()
	{
		global $bb_cfg;
		
		//$request = new Zend_Controller_Request_Http();
		
		$url = self::requestUrl();
		$match = preg_split('#/(.*?)#', $url);
		$match = array_filter($match);	
		self::redirect($match);
		
		$mode = (isset($match[1])) ? self::detectedMode($match[1]) : '';
		$id = (isset($match[2])) ? self::detectedId($match[2]) : '';

		if(!defined('SEO_ID')) define('SEO_ID', $id);
		if(!defined('SEO_MODE')) define('SEO_MODE', $mode);
		
		if($mode == 'settings') $def_mode = 'register';
		if($mode == 'category') $def_mode = 'index';	
			
		if(in_array($mode, $bb_cfg['fulllink']['ucp'])) $ucp = 'ucp/';
		else $ucp = '';	
		
		$patch = BB_ROOT .'library/page/';
		$url = $patch.self::$index;
		
		if(!$mode) {
			$define = 'index';
		} else {
			$modes = (isset($def_mode)) ? $def_mode : $mode;	
			$url = $patch.$ucp.$modes.'.php';
			if(file_exists($url)) {
				$define = ($mode && $mode != 'category') ? $mode : 'index';
			} else {
				$define = 'error';
				$url = $patch.'error.php';	
			}
		}
		
		if(!defined('BB_SCRIPT')) define('BB_SCRIPT', $define);
		
		return $url;	
	}
	
	private static function detectedMode($mode)
	{
		global $bb_cfg;
		
		$mode = self::replaceUrl($mode, array(self::$index => '', '?' => ''));

		return (string) $mode;	
	}
	
	private static function detectedId($match) 
	{
		if(isset($match)) {
			$mode = explode('.', $match);
			return (isset($mode[1])) ? intval($mode[1]) : intval($match);
		}
	}
	
	private static function redirect($match) 
	{
		global $bb_cfg;

		if($bb_cfg['no_rewite_url']) {	
			$link = self::replaceUrl(self::requestUrl(), array(self::$index => '', self::$index.'?' => ''));
			if(($link != self::requestUrl() || (mb_substr(self::requestUrl(), -1) != '/')) && !isset($match[3])) self::redirectRequestUrl(make_url(substr($link, 1)).'/');	
		}
	}
	
	public static function canonicalizeRequestUrl($linkUrl)
	{

		global $bb_cfg;
	
		$linkUrl = strval($linkUrl);
		$requestUri = self::requestUrl();
		$parts = explode('?', $requestUri);
		$rew = ($bb_cfg['no_rewite_url']) ? $parts[0] : $parts[0].'?'.$parts[1];
		$prefix = (isset($parts[2])) ? $parts[2] : (isset($parts[1]) ? $parts[1] : '');
		$requestUrlPrefix = isset($rew) ? substr($rew, 1) : '';
		$link = ($prefix) ? make_url($linkUrl.'?'.$prefix): make_url($linkUrl).'/';

		if($requestUrlPrefix != $linkUrl) self::redirectRequestUrl($link);
	}

	
	public static function redirectRequestUrl($link)
	{
		header( 'Location: '.$link, true, 301 );
	}

	public static function requestUrlNotfound()
	{
		header("HTTP/1.1 404 Not Found"); 
	}
}